<?php

namespace WPStaging\Backend\Pro\Modules\Jobs;

use WPStaging\Backend\Modules\Jobs\JobExecutable;
use WPStaging\Core\WPStaging;
use WPStaging\Framework\Database\TablesRenamer;

/**
 * @package WPStaging\Backend\Modules\Jobs
 */
class DatabaseTmpRename extends JobExecutable
{
    /**
     * @var TablesRenamer
     */
    private $tablesRenamer;

    /** @var string */
    private $productionTablePrefix;

    /**
     * Initialize
     */
    public function initialize()
    {
        $wpdb                        = WPStaging::make('wpdb');
        $this->productionTablePrefix = $wpdb->prefix;

        $this->checkFatalError();
        $this->tablesRenamer = WPStaging::make(TablesRenamer::class);
    }

    protected function checkFatalError()
    {
        if (DatabaseTmp::TMP_PREFIX === $this->productionTablePrefix) {
            $this->returnException('Fatal Error: Prefix ' . $this->productionTablePrefix . ' is used for the live site and used for the temporary database tables hence we can not replace the production database. Please ask support@wp-staging.com how to resolve this.');
        }
    }

    /**
     * Calculate Total Steps in This Job and Assign It to $this->options->totalSteps
     * @return void
     */
    protected function calculateTotalSteps()
    {
        $this->options->totalSteps = 3;
    }

    /**
     * Execute the Current Step
     * Returns false when over threshold limits are hit or when the job is done, true otherwise
     * @return bool
     * @throws \Exception
     */
    protected function execute()
    {
        // Over limits threshold
        if ($this->isOverThreshold()) {
            $this->log('DB Rename: Is over threshold. Continuing ...');
            // Prepare response and save current progress
            $this->prepareResponse(false, false);
            $this->saveOptions();
            return false;
        }

        if ($this->options->currentStep === 0) {
            $this->preDatabaseRenameActions();
            $this->prepareResponse(false, true);
            return false;
        }

        if ($this->options->currentStep === 1) {
            $incrementStep = $this->performDatabaseRename();
            $this->prepareResponse(false, $incrementStep);
            return false;
        }

        if ($this->options->currentStep === 2) {
            $this->postDatabaseRenameActions();
            $this->prepareResponse(false, true);
            return false;
        }

        $this->isFinished();
        $this->prepareResponse(true, false);
        return false;
    }

    protected function setupTableRenamer()
    {
        $this->tablesRenamer->setTmpPrefix(DatabaseTmp::TMP_PREFIX);
        $this->tablesRenamer->setProductionTablePrefix($this->productionTablePrefix);
        $this->tablesRenamer->setDropPrefix(DatabaseTmp::TMP_PREFIX_TO_DROP);
        $this->tablesRenamer->setShortNamedTablesToRename([]);
        $this->tablesRenamer->setShortNamedTablesToDrop([]);
        $this->tablesRenamer->setRenameViews(false);
        $this->tablesRenamer->setExcludedTables([]);
        $this->tablesRenamer->setLogEachRename(true);
        $this->tablesRenamer->setLogger($this->logger);
        $this->tablesRenamer->setThresholdCallable([$this, 'isOverThreshold']);

        $this->tablesRenamer->setupRenamer();
    }

    protected function preDatabaseRenameActions()
    {
        $this->setupTableRenamer();
        $this->options->totalTablesToRename     = $this->tablesRenamer->getTotalTables();
        $this->options->totalTablesRenamed      = 0;
        $this->options->activePluginsToPreserve = $this->tablesRenamer->getActivePluginsToPreserve();
        $this->options->isNetworkActivePlugin   = is_plugin_active_for_network(WPSTG_PLUGIN_FILE);

        if ($this->options->isNetworkActivePlugin && $this->options->isNetworkClone) {
            $this->options->activeSitewidePluginsToPreserve = $this->tablesRenamer->getActiveSitewidePluginsToPreserve();
        }

        if (!empty($this->options->activePluginsToPreserve)) {
            $this->log('DB Rename: Preserved Active Plugins');
        }
    }

    /** @return bool */
    protected function performDatabaseRename()
    {
        $this->setupTableRenamer();

        $result = $this->tablesRenamer->renameNonConflictingTables();
        if ($result === false) {
            $tablesRenamed = $this->options->totalTablesRenamed + $this->tablesRenamer->getRenamedTables();
            $this->log(sprintf('DB Rename: Renamed %d/%d tables', $tablesRenamed, $this->options->totalTablesToRename));
            $this->options->totalTablesRenamed = $tablesRenamed;
            return false;
        }

        $result = $this->tablesRenamer->renameConflictingTables();
        $tablesRenamed = $this->options->totalTablesRenamed + $this->tablesRenamer->getRenamedTables();
        $this->log(sprintf('DB Rename: Renamed %d/%d tables', $tablesRenamed, $this->options->totalTablesToRename));
        $this->options->totalTablesRenamed = $tablesRenamed;
        return $result;
    }

    protected function postDatabaseRenameActions()
    {
        $activeWpstgPlugin = plugin_basename(trim(WPSTG_PLUGIN_FILE));
        if (!empty($this->options->activePluginsToPreserve)) {
            $this->tablesRenamer->setProductionTablePrefix($this->productionTablePrefix);
            $this->tablesRenamer->restorePreservedActivePlugins($this->options->activePluginsToPreserve, $activeWpstgPlugin, false);
            $this->log('DB Rename: Restored Preserved Active Plugins');
        }

        if ($this->options->isNetworkActivePlugin && $this->options->isNetworkClone) {
            $this->tablesRenamer->restorePreservedActiveSitewidePlugins($this->options->isNetworkActivePlugin, $activeWpstgPlugin);
            $this->log('DB Rename: Preserved Network Active Plugin Status');
        }

        $this->flush();
    }

    /**
     * Flush wpdb cache and permalinks rewrite rules
     * to prevent 404s and other oddities
     * @global object $wp_rewrite
     */
    protected function flush()
    {
        wp_cache_flush();
        global $wp_rewrite;
        $wp_rewrite->init();
        flush_rewrite_rules(true); // true = hard refresh, recreates the .htaccess file
    }

    /**
     * Push is finished
     * @return bool
     */
    protected function isFinished()
    {
        // This job is finished
        if ($this->options->currentStep >= $this->options->totalSteps) {
            $this->log('DB Rename: Has been finished successfully. Cleaning up...');
            $this->prepareResponse(true, false);
            return true;
        }

        return false;
    }
}
